package ru.landgrafhomyak.serdha.api.v0.queries

import ru.landgrafhomyak.serdha.api.v0.misc.Expression
import ru.landgrafhomyak.serdha.api.v0.module.CreateModuleScope
import ru.landgrafhomyak.serdha.api.v0.table.Table
import ru.landgrafhomyak.serdha.api.v0.misc.DatabaseType
import ru.landgrafhomyak.serdha.api.v0.misc.SelectedTable

public interface DeleteQuery<QueryUserExtension : Any> : _CommonCreateQueryScope.CanBeSubquery<QueryUserExtension> {
	public val userExtension: QueryUserExtension

	/**
	 * Functional interface for creating 'DELETE' query.
	 *
	 * Can be implemented like a companion object that just calls [QueryUserExtension]'s constructor with the same or similar signature.
	 *
	 * @param QueryUserExtension User's type with descriptors related to query.
	 * @param TargetTableUserExtension User's type with descriptors related to table from which data will be deleted.
	 *
	 * @see CreateModuleScope.createDeleteQuery
	 * @see CreateModuleScope.createTemporaryDeleteQuery
	 */
	public interface Creator<QueryUserExtension : Any, TargetTableUserExtension : Any> {
		/**
		 * Scope method that uses [creator] to create a query.
		 *
		 * @param table Descriptor of table from which data will be deleted.
		 * @param creator Object with descriptor providers and factories.
		 * @return User's object with query-related descriptors for future access.
		 *
		 * @see CreateModuleScope.createDeleteQuery
		 * @see CreateModuleScope.createTemporaryDeleteQuery
		 */
		public fun createDelete(table: Table<TargetTableUserExtension, *>, creator: Scope<QueryUserExtension, TargetTableUserExtension>): QueryUserExtension

		public interface Scope<QueryUserExtension : Any, TargetTableUserExtension : Any> : _CommonCreateQueryScope<QueryUserExtension> {
			public val targetTable: SelectedTable<TargetTableUserExtension, QueryUserExtension>

			public fun where(expression: Expression<Boolean, DatabaseType.BOOLEAN, QueryUserExtension>)

			public fun limit(size: UInt)

			public val returning: _CommonCreateQueryScope._ReturningClauseScope<QueryUserExtension>
		}
	}
}