package ru.landgrafhomyak.serdha.api.v0.queries

import ru.landgrafhomyak.serdha.api.v0.misc.Expression
import ru.landgrafhomyak.serdha.api.v0.module.CreateModuleScope
import ru.landgrafhomyak.serdha.api.v0.misc.Column
import ru.landgrafhomyak.serdha.api.v0.table.Table
import ru.landgrafhomyak.serdha.api.v0.misc.DatabaseType
import ru.landgrafhomyak.serdha.api.v0.misc.InputParam

public interface InsertQuery<QueryUserExtension : Any> {
	public val userExtension: QueryUserExtension

	/**
	 * Functional interface for creating 'INSERT ... VALUES' query.
	 *
	 * Can be implemented like a companion object that just calls [QueryUserExtension]'s constructor with the same or similar signature.
	 *
	 * @param QueryUserExtension User's type with descriptors related to query.
	 * @param TargetTableUserExtension User's type with descriptors related to table to which values will be inserted.
	 *
	 * @see CreateModuleScope.createInsertQuery
	 * @see CreateModuleScope.createTemporaryInsertQuery
	 */
	public interface Creator<QueryUserExtension : Any, TargetTableUserExtension : Any> {
		/**
		 * Scope method that uses [creator] to create a query.
		 *
		 * @param table Descriptor of table to which values will be inserted.
		 * @param creator Object with descriptor providers and factories.
		 * @return User's object with query-related descriptors for future access.
		 *
		 * @see CreateModuleScope.createInsertQuery
		 * @see CreateModuleScope.createTemporaryInsertQuery
		 */
		public fun createInsert(table: Table<TargetTableUserExtension, *>, creator: Scope<QueryUserExtension, TargetTableUserExtension>): QueryUserExtension

		public interface Scope<QueryUserExtension : Any, TargetTableUserExtension : Any> : _CreateInsertOrCopyQueryScope<QueryUserExtension, TargetTableUserExtension> {
			public class DataParam<@Suppress("unused") QueryUserExtension : Any> private constructor()

			public val dataExpressionBuilder: Expression.Builder<DataParam<QueryUserExtension>>

			public fun <RT, DT : DatabaseType<RT>> dataParam(name: String, type: DT): InputParam<RT, DT, DataParam<QueryUserExtension>>

			public fun <RT, DT : DatabaseType<RT>> insertParam(
				column: Column<RT, DT, TargetTableUserExtension>,
				paramName: String = column.name,
			): InputParam<RT, DT, DataParam<QueryUserExtension>>

			public fun <RuntimeType, DT : DatabaseType<RuntimeType>> insert(column: Column<RuntimeType, DT, TargetTableUserExtension>, expression: Expression<RuntimeType, DT, DataParam<QueryUserExtension>>)
		}
	}
}