package ru.landgrafhomyak.serdha.api.v0.queries

import ru.landgrafhomyak.serdha.api.v0.misc.Expression
import ru.landgrafhomyak.serdha.api.v0.misc.DatabaseType
import ru.landgrafhomyak.serdha.api.v0.module.CreateModuleScope

public interface SelectQuery<QueryUserExtension : Any> : _CommonCreateQueryScope.CanBeSubquery<QueryUserExtension> {
	public val userExtension: QueryUserExtension

	public enum class Order {
		ASC, DESC
	}

	/**
	 * Functional interface for creating 'SELECT' query.
	 *
	 * Can be implemented like a companion object that just calls [QueryUserWrapper]'s constructor with the same or similar signature.
	 *
	 * @param QueryUserWrapper User's type with descriptors related to query.
	 *
	 * @see CreateModuleScope.createSelectQuery
	 * @see CreateModuleScope.createTemporarySelectQuery
	 */
	public interface Creator<QueryUserWrapper : Any> {
		/**
		 * Scope method that uses [creator] to create a query.
		 *
		 * @param creator Object with descriptor providers and factories.
		 * @return User's object with query-related descriptors for future access.
		 *
		 * @see CreateModuleScope.createSelectQuery
		 * @see CreateModuleScope.createTemporarySelectQuery
		 */
		public fun createSelect(creator: Scope<QueryUserWrapper>): QueryUserWrapper

		public interface Scope<QueryUserExtension : Any> : _CommonCreateQueryScope<QueryUserExtension>, _CommonCreateQueryScope._ReturningClauseScope<QueryUserExtension> {
			public fun where(expression: Expression<Boolean, DatabaseType.BOOLEAN, QueryUserExtension>)

			public fun limit(size: UInt)

			@Suppress("SpellCheckingInspection")
			public fun offsetedLimit(offset: ULong, size: UInt)
		}
	}
}