package ru.landgrafhomyak.serdha.api.v0.queries

import ru.landgrafhomyak.serdha.api.v0.misc.Expression
import ru.landgrafhomyak.serdha.api.v0.module.CreateModuleScope
import ru.landgrafhomyak.serdha.api.v0.misc.Column
import ru.landgrafhomyak.serdha.api.v0.table.Table
import ru.landgrafhomyak.serdha.api.v0.misc.DatabaseType
import ru.landgrafhomyak.serdha.api.v0.misc.SelectedTable

public interface UpdateQuery<QueryUserExtension : Any> : _CommonCreateQueryScope.CanBeSubquery<QueryUserExtension> {
	public val userExtension: QueryUserExtension

	/**
	 * Functional interface for creating 'UPDATE' query.
	 *
	 * Can be implemented like a companion object that just calls [QueryUserWrapper]'s constructor with the same or similar signature.
	 *
	 * @param QueryUserWrapper User's type with descriptors related to query.
	 * @param TableUserExtension User's type with descriptors related to table, whose rows will be updated.
	 *
	 * @see CreateModuleScope.createUpdateQuery
	 * @see CreateModuleScope.createTemporaryUpdateQuery
	 */
	public interface Creator<QueryUserWrapper : Any, TargetTableUserExtension : Any> {
		/**
		 * Scope method that uses [creator] to create a query.
		 *
		 * @param table Descriptor of table which rows will be updated.
		 * @param creator Object with descriptor providers and factories.
		 * @return User's object with query-related descriptors for future access.
		 *
		 * @see CreateModuleScope.createUpdateQuery
		 * @see CreateModuleScope.createTemporaryUpdateQuery
		 */
		public fun createUpdate(table: Table<TargetTableUserExtension, *>, creator: Scope<QueryUserWrapper, TargetTableUserExtension>): QueryUserWrapper

		public interface Scope<QueryUserExtension : Any, TargetTableUserExtension : Any> : _CommonCreateQueryScope<QueryUserExtension> {
			public val targetTable: SelectedTable<TargetTableUserExtension, QueryUserExtension>

			public fun <RuntimeType, DT : DatabaseType<RuntimeType>> selectOldColumnValue(c: Column<RuntimeType, DT, TargetTableUserExtension>): Expression<RuntimeType, DT, QueryUserExtension>

			public fun <RuntimeType, DT : DatabaseType<RuntimeType>> updateColumn(c: Column<RuntimeType, DT, TargetTableUserExtension>, e: Expression<RuntimeType, DT, QueryUserExtension>)

			public fun where(expression: Expression<Boolean, DatabaseType.BOOLEAN, QueryUserExtension>)

			public fun limit(size: UInt)

			@Suppress("SpellCheckingInspection")
			public fun offsetedLimit(offset: ULong, size: UInt)

			public val returning: _CommonCreateQueryScope._ReturningClauseScope<QueryUserExtension>
		}
	}
}